'' =================================================================================================
''
''   File....... hc-8_candles.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" Williams, EFX-TEK
''               Copyright (c) 2011-2012 EFX-TEK
''               All Rights Reserved
''   E-mail..... teamefx@efx-tek.com
''   Started.... 
''   Updated.... 23 MAR 2012
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000
  

con

  PCB_REV = "C"                                                 ' production boards

  RX1     = 31                                                  ' programming / debug port
  TX1     = 30
  
  SDA     = 29                                                  ' eeprom / i2c
  SCL     = 28

  OUT_EN  = 27                                                  ' output enable     
  
  SIO     = 26                                                  ' TTL serial io

  NC25    = 25                                                  ' not connected
  NC24    = 24
  
  RX2     = 23                                                  ' RS-485 interface
  RX2_EN  = 22
  TX2_EN  = 21
  TX2     = 20

  NC19    = 19
  
  R_LED   = 18                                                  ' bi-color LED
  G_LED   = 17

  NC16    = 16
  
  OUT7    = 15                                                  ' channel outputs
  OUT6    = 14   
  OUT5    = 13 
  OUT4    = 12
  OUT3    = 11
  OUT2    = 10   
  OUT1    =  9 
  OUT0    =  8
          
  OPT_BR  =  7                                                  ' option inputs
  OPT_A1  =  6   
  OPT_A0  =  5
  OPT_SM  =  4

  DMX_A8  =  3                                                  ' DMX address.8 input

  DO_165  =  2                                                  ' 74x165 IO
  CLK_165 =  1 
  LD_165  =  0


  ' for EFX-TEK uSD adapter module
  ' -- must open all option switches when installed

  SD_CS   =  OPT_BR                                       
  SD_DI   =  OPT_A1  
  SD_CLK  =  OPT_A0  
  SD_DO   =  OPT_SM  


con

   #0, NO, YES
   #0, OFF, RED, GRN


obj

  rr   : "realrandom"                                           ' random from hardware
  prng : "efx_prng"                                             ' advanced pseudo-random
  leds : "efx_pwm8"                                             ' pwm output for LEDs


var

  long  target[8]                                               ' target for each channel
  

pub main | ch, level, t  

  outa[OUT7..OUT0] := %0000_0000                                ' clear outputs
  dira[OUT7..OUT0] := %1111_1111                                ' output mode                                  
  low(OUT_EN)                                                   ' enable 74x245

  ' setup randomizer
  ' -- use hardware to seed advanced pseudo-random object
  ' -- excellent random without extra cog (loaded and then released)
                                                     
  rr.start                                                      
  prng.seed(rr.random, rr.random, rr.random, rr.random, rr.random) 
  rr.stop
                                                       
  leds.start(8, OUT0)                                           ' start LED dimmer

  repeat ch from 0 to 7                                         ' seed the wicks
    leds.set(ch, prng.random // 55 + 201)                       ' light the led
    target[ch] := prng.random // 55 + 201                       ' set 1st target

  t := cnt                                                      ' synchronize loop timer
  repeat
    repeat ch from 0 to 7
      if (leds.read(ch) == target[ch])                          ' at target?
        target[ch] := prng.random // 55 + 201                   ' yes, set new one
      elseif (leds.read(ch) < target[ch])                       ' below target?
        leds.inc(ch)                                            ' yes, bump up
      else
        leds.dec(ch)                                            ' no, bump down

    ' minimum delay is 1400us
        
    waitcnt(t += (2_500 * US_001))                              ' run every 2.5ms


con

  { ================================= }
  {                                   }
  {  S U P P O R T   R O U T I N E S  } 
  {                                   }
  { ================================= }


pub pause(ms) | t

'' Delay program ms milliseconds

  t := cnt - 1088                                               ' sync with system counter
  repeat (ms #> 0)                                              ' delay must be > 0
    waitcnt(t += MS_001)

  
pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin, refreshes output mode

  !outa[pin]
  dira[pin] := 1
  

pub input(pin)

'' Set pin to input mode and read back current state

  dira[pin] := 0

  return ina[pin]


dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      